/* USER CODE BEGIN Header */
/**
  ******************************************************************************
  * @file           : main.c
  * @brief          : Main program body
  ******************************************************************************
  * @attention
  *
  * Copyright (c) 2022 STMicroelectronics.
  * All rights reserved.
  *
  * This software is licensed under terms that can be found in the LICENSE file
  * in the root directory of this software component.
  * If no LICENSE file comes with this software, it is provided AS-IS.
  *
  ******************************************************************************
  */
/* USER CODE END Header */
/* Includes ------------------------------------------------------------------*/
#include "main.h"
#include "usb_host.h"

/* Private includes ----------------------------------------------------------*/
/* USER CODE BEGIN Includes */
typedef
		struct	{
	uint8_t			address;	// アドレスNo.
	uint32_t		bit;		// ビット位置
}
KEYB_TABLE;

#include "usbh_hid_gamepad.h"
#include "usbh_hid.h"
#include "usb_keytable.h"
/* USER CODE END Includes */

/* Private typedef -----------------------------------------------------------*/
/* USER CODE BEGIN PTD */
typedef
		enum
{
	KEYBOARD_MODE = 0,
	GAMEPAD_MODE,
	MOUSE_MODE
}
PC60_MODE;
/* USER CODE END PTD */

/* Private define ------------------------------------------------------------*/
/* USER CODE BEGIN PD */
#define KEYB_CTRL	   		(0xe0)
#define KEYB_SHIFT	   		(0xe1)
#define KEYB_GRAPH	  	 	(0x8a)

#define	 KEYB_UP			(0x52)
#define	 KEYB_DOWN			(0x51)
#define	 KEYB_LEFT			(0x50)
#define	 KEYB_RIGHT			(0x4f)
#define	 KEYB_SPACE			(0x2c)
#define	 KEYB_RET			(0x28)
#define KEYB_Y				(0x1c)

#define HATSW_UP           (0)
#define HATSW_UPRIGHT      (1)
#define HATSW_RIGHT        (2)
#define HATSW_DOWNRIGHT    (3)
#define HATSW_DOWN         (4)
#define HATSW_DOWNLEFT     (5)
#define HATSW_LEFT         (6)
#define HATSW_UPLEFT       (7)

#define RAPID_MAX          (30000)

#define	 ADDRESS_MAX		(0x10)

#define MOUSE_UP		(0xF8)
#define MOUSE_CENTER	(0x80)
#define MOUSE_DOWN		(0x08)

/* USER CODE END PD */

/* Private macro -------------------------------------------------------------*/
/* USER CODE BEGIN PM */

/* USER CODE END PM */

/* Private variables ---------------------------------------------------------*/
TIM_HandleTypeDef htim11;

/* USER CODE BEGIN PV */
static KEYB_TABLE	usb_keytable[USB_SIZE];
uint32_t			pc60_data[ADDRESS_MAX] = {0};
uint32_t			pc60_data_4bit[ADDRESS_MAX] = {0};

uint32_t			pc60_reset = 0x00000000;

uint32_t			*pc60_data0  = &pc60_data[PC60_ID_Y0];
uint32_t			*pc60_data1  = &pc60_data[PC60_ID_Y1];
uint32_t			*pc60_data2  = &pc60_data[PC60_ID_Y2];
uint32_t			*pc60_data3  = &pc60_data[PC60_ID_Y3];
uint32_t			*pc60_data4  = &pc60_data[PC60_ID_Y4];
uint32_t			*pc60_data5  = &pc60_data[PC60_ID_Y5];
uint32_t			*pc60_data6  = &pc60_data[PC60_ID_Y6];
uint32_t			*pc60_data7  = &pc60_data[PC60_ID_Y7];
uint32_t			*pc60_data8  = &pc60_data[PC60_ID_Y8];
uint32_t			*pc60_data9  = &pc60_data[PC60_ID_Y9];

// GamePad
static uint8_t		pad_con_x     = 0;
static uint8_t		pad_con_y     = 0;
static uint8_t		pad_con_hatsw = 0;
static uint8_t		pad_bot_a     = 0;
static uint8_t		pad_bot_b     = 0;
static uint8_t		pad_bot_c     = 0;
static uint8_t		pad_bot_x     = 0;
static uint8_t		pad_bot_y     = 0;
static uint8_t		pad_bot_z     = 0;
static uint8_t		pad_bot_s     = 0;
static uint8_t		pad_bot_m     = 0;

static uint32_t		rapid_count = 0;
static PC60_MODE	pc60_mode = KEYBOARD_MODE;

/* USER CODE END PV */

/* Private function prototypes -----------------------------------------------*/
void SystemClock_Config(void);
static void MX_GPIO_Init(void);
static void MX_TIM11_Init(void);
void MX_USB_HOST_Process(void);

/* USER CODE BEGIN PFP */
static void set_usb_mouse(USBH_HandleTypeDef *phost);
static void set_usb_keycode(USBH_HandleTypeDef *phost);
static void set_usb_gamepad(USBH_HandleTypeDef *phost);
static void pc60_gamepad(void);
/* USER CODE END PFP */

/* Private user code ---------------------------------------------------------*/
/* USER CODE BEGIN 0 */

/* USER CODE END 0 */

/**
  * @brief  The application entry point.
  * @retval int
  */
int main(void)
{
  /* USER CODE BEGIN 1 */

  /* USER CODE END 1 */

  /* MCU Configuration--------------------------------------------------------*/

  /* Reset of all peripherals, Initializes the Flash interface and the Systick. */
  HAL_Init();

  /* USER CODE BEGIN Init */

  /* USER CODE END Init */

  /* Configure the system clock */
  SystemClock_Config();

  /* USER CODE BEGIN SysInit */

  /* USER CODE END SysInit */

  /* Initialize all configured peripherals */
  MX_GPIO_Init();
  MX_USB_HOST_Init();
  MX_TIM11_Init();
  /* USER CODE BEGIN 2 */
	// VBUS Disabled
	USB_OTG_FS->GCCFG |= USB_OTG_GCCFG_NOVBUSSENS;

	// JTAG Disabled
	GPIOB->MODER = (GPIOB->MODER & 0xFFFFF03F) | 0x00000000;

	pc60_reset = (uint32_t)(PC60_X0_Pin|PC60_X1_Pin|PC60_X2_Pin|PC60_X3_Pin|PC60_X4_Pin|PC60_X5_Pin|PC60_X6_Pin|PC60_X7_Pin);

	// ビットシフト時間短縮のため事前に並べ替えておく
	for( uint32_t index = 0; index < USB_SIZE; index++ ){
		usb_keytable[index].address = pc60_usb_keytable[index].address;
		usb_keytable[index].bit = 0x00000000;
		usb_keytable[index].bit |= (pc60_usb_keytable[index].bit & 0x01 ? ((uint32_t)PC60_X0_Pin << 16u) : 0x00000000);
		usb_keytable[index].bit |= (pc60_usb_keytable[index].bit & 0x02 ? ((uint32_t)PC60_X1_Pin << 16u) : 0x00000000);
		usb_keytable[index].bit |= (pc60_usb_keytable[index].bit & 0x04 ? ((uint32_t)PC60_X2_Pin << 16u) : 0x00000000);
		usb_keytable[index].bit |= (pc60_usb_keytable[index].bit & 0x08 ? ((uint32_t)PC60_X3_Pin << 16u) : 0x00000000);
		usb_keytable[index].bit |= (pc60_usb_keytable[index].bit & 0x10 ? ((uint32_t)PC60_X4_Pin << 16u) : 0x00000000);
		usb_keytable[index].bit |= (pc60_usb_keytable[index].bit & 0x20 ? ((uint32_t)PC60_X5_Pin << 16u) : 0x00000000);
		usb_keytable[index].bit |= (pc60_usb_keytable[index].bit & 0x40 ? ((uint32_t)PC60_X6_Pin << 16u) : 0x00000000);
		usb_keytable[index].bit |= (pc60_usb_keytable[index].bit & 0x80 ? ((uint32_t)PC60_X7_Pin << 16u) : 0x00000000);
	}

	HAL_Delay(500);

	// タイマ開始
	HAL_TIM_Base_Start_IT(&htim11);
  /* USER CODE END 2 */

  /* Infinite loop */
  /* USER CODE BEGIN WHILE */
  while (1)
  {
    /* USER CODE END WHILE */
    MX_USB_HOST_Process();

    /* USER CODE BEGIN 3 */
  }
  /* USER CODE END 3 */
}

/**
  * @brief System Clock Configuration
  * @retval None
  */
void SystemClock_Config(void)
{
  RCC_OscInitTypeDef RCC_OscInitStruct = {0};
  RCC_ClkInitTypeDef RCC_ClkInitStruct = {0};

  /** Configure the main internal regulator output voltage
  */
  __HAL_RCC_PWR_CLK_ENABLE();
  __HAL_PWR_VOLTAGESCALING_CONFIG(PWR_REGULATOR_VOLTAGE_SCALE2);

  /** Initializes the RCC Oscillators according to the specified parameters
  * in the RCC_OscInitTypeDef structure.
  */
  RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_HSE;
  RCC_OscInitStruct.HSEState = RCC_HSE_ON;
  RCC_OscInitStruct.PLL.PLLState = RCC_PLL_ON;
  RCC_OscInitStruct.PLL.PLLSource = RCC_PLLSOURCE_HSE;
  RCC_OscInitStruct.PLL.PLLM = 25;
  RCC_OscInitStruct.PLL.PLLN = 336;
  RCC_OscInitStruct.PLL.PLLP = RCC_PLLP_DIV4;
  RCC_OscInitStruct.PLL.PLLQ = 7;
  if (HAL_RCC_OscConfig(&RCC_OscInitStruct) != HAL_OK)
  {
    Error_Handler();
  }

  /** Initializes the CPU, AHB and APB buses clocks
  */
  RCC_ClkInitStruct.ClockType = RCC_CLOCKTYPE_HCLK|RCC_CLOCKTYPE_SYSCLK
                              |RCC_CLOCKTYPE_PCLK1|RCC_CLOCKTYPE_PCLK2;
  RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_PLLCLK;
  RCC_ClkInitStruct.AHBCLKDivider = RCC_SYSCLK_DIV1;
  RCC_ClkInitStruct.APB1CLKDivider = RCC_HCLK_DIV2;
  RCC_ClkInitStruct.APB2CLKDivider = RCC_HCLK_DIV1;

  if (HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_2) != HAL_OK)
  {
    Error_Handler();
  }
}

/**
  * @brief TIM11 Initialization Function
  * @param None
  * @retval None
  */
static void MX_TIM11_Init(void)
{

  /* USER CODE BEGIN TIM11_Init 0 */

  /* USER CODE END TIM11_Init 0 */

  /* USER CODE BEGIN TIM11_Init 1 */

  /* USER CODE END TIM11_Init 1 */
  htim11.Instance = TIM11;
  htim11.Init.Prescaler = 83;
  htim11.Init.CounterMode = TIM_COUNTERMODE_UP;
  htim11.Init.Period = 33333;
  htim11.Init.ClockDivision = TIM_CLOCKDIVISION_DIV1;
  htim11.Init.AutoReloadPreload = TIM_AUTORELOAD_PRELOAD_ENABLE;
  if (HAL_TIM_Base_Init(&htim11) != HAL_OK)
  {
    Error_Handler();
  }
  /* USER CODE BEGIN TIM11_Init 2 */

  /* USER CODE END TIM11_Init 2 */

}

/**
  * @brief GPIO Initialization Function
  * @param None
  * @retval None
  */
static void MX_GPIO_Init(void)
{
  GPIO_InitTypeDef GPIO_InitStruct = {0};

  /* GPIO Ports Clock Enable */
  __HAL_RCC_GPIOH_CLK_ENABLE();
  __HAL_RCC_GPIOA_CLK_ENABLE();
  __HAL_RCC_GPIOB_CLK_ENABLE();

  /*Configure GPIO pin Output Level */
  HAL_GPIO_WritePin(GPIOA, PC60_X0_Pin|PC60_X1_Pin|PC60_X2_Pin|PC60_X3_Pin
                          |PC60_X4_Pin|PC60_X5_Pin|PC60_X6_Pin|PC60_X7_Pin, GPIO_PIN_SET);

  /*Configure GPIO pins : PC60_X0_Pin PC60_X1_Pin PC60_X2_Pin PC60_X3_Pin
                           PC60_X4_Pin PC60_X5_Pin PC60_X6_Pin PC60_X7_Pin */
  GPIO_InitStruct.Pin = PC60_X0_Pin|PC60_X1_Pin|PC60_X2_Pin|PC60_X3_Pin
                          |PC60_X4_Pin|PC60_X5_Pin|PC60_X6_Pin|PC60_X7_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_OD;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_VERY_HIGH;
  HAL_GPIO_Init(GPIOA, &GPIO_InitStruct);

  /*Configure GPIO pins : PC60_Y5_Pin PC60_Y6_Pin PC60_Y7_Pin PC60_Y1_Pin
                           PC60_Y2_Pin PC60_Y3_Pin PC60_Y4_Pin PC60_Y8_Pin
                           PC60_Y9_Pin */
  GPIO_InitStruct.Pin = PC60_Y5_Pin|PC60_Y6_Pin|PC60_Y7_Pin|PC60_Y1_Pin
                          |PC60_Y2_Pin|PC60_Y3_Pin|PC60_Y4_Pin|PC60_Y8_Pin
                          |PC60_Y9_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_IT_FALLING;
  GPIO_InitStruct.Pull = GPIO_PULLUP;
  HAL_GPIO_Init(GPIOB, &GPIO_InitStruct);

  /*Configure GPIO pin : PC60_A_Pin */
  GPIO_InitStruct.Pin = PC60_A_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_IT_RISING;
  GPIO_InitStruct.Pull = GPIO_PULLDOWN;
  HAL_GPIO_Init(PC60_A_GPIO_Port, &GPIO_InitStruct);

  /*Configure GPIO pins : PC60_A0_Pin PC60_A1_Pin PC60_A2_Pin PC60_A3_Pin */
  GPIO_InitStruct.Pin = PC60_A0_Pin|PC60_A1_Pin|PC60_A2_Pin|PC60_A3_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_IT_FALLING;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  HAL_GPIO_Init(GPIOB, &GPIO_InitStruct);

  /* EXTI interrupt init*/
  HAL_NVIC_SetPriority(EXTI0_IRQn, 1, 0);
  HAL_NVIC_EnableIRQ(EXTI0_IRQn);

  HAL_NVIC_SetPriority(EXTI1_IRQn, 1, 0);
  HAL_NVIC_EnableIRQ(EXTI1_IRQn);

  HAL_NVIC_SetPriority(EXTI2_IRQn, 1, 0);
  HAL_NVIC_EnableIRQ(EXTI2_IRQn);

  HAL_NVIC_SetPriority(EXTI3_IRQn, 1, 0);
  HAL_NVIC_EnableIRQ(EXTI3_IRQn);

  HAL_NVIC_SetPriority(EXTI4_IRQn, 1, 0);
  HAL_NVIC_EnableIRQ(EXTI4_IRQn);

  HAL_NVIC_SetPriority(EXTI9_5_IRQn, 1, 0);
  HAL_NVIC_EnableIRQ(EXTI9_5_IRQn);

  HAL_NVIC_SetPriority(EXTI15_10_IRQn, 0, 0);
  HAL_NVIC_EnableIRQ(EXTI15_10_IRQn);

}

/* USER CODE BEGIN 4 */

/**
 * @brief  Period elapsed callback in non-blocking mode
 * @param  htim TIM handle
 * @retval None
 */
void HAL_TIM_PeriodElapsedCallback(TIM_HandleTypeDef *htim)
{
	/* Prevent unused argument(s) compilation warning */
	//  UNUSED(htim);

	/* NOTE : This function should not be modified, when the callback is needed,
            the HAL_TIM_PeriodElapsedCallback could be implemented in the user file
	 */
	if (htim == &htim11){
		pc60_gamepad();
	}
}

/**
 * @brief  The function is a callback about HID Data events
 *  @param  phost: Selected device
 * @retval None
 */
void USBH_HID_EventCallback(USBH_HandleTypeDef *phost)
{
	/* Prevent unused argument(s) compilation warning */
	//  UNUSED(phost);
	HID_TypeTypeDef			hid_type = USBH_HID_GetDeviceType(phost);

	// Get HID Mouse Information
	if(hid_type == HID_MOUSE){
		pc60_mode = MOUSE_MODE;
		set_usb_mouse(phost);
	}
	// Get HID Keyboard Information
	else if(hid_type == HID_KEYBOARD){
		pc60_mode = KEYBOARD_MODE;
		set_usb_keycode(phost);
	}
	// Get HID Gamepad Information
	else{
		pc60_mode = GAMEPAD_MODE;
		set_usb_gamepad(phost);
	}
}

static void set_usb_mouse(USBH_HandleTypeDef *phost)
{
	HID_MOUSE_Info_TypeDef	*mouse_info = USBH_HID_GetMouseInfo(phost);
	uint8_t mouse_x = mouse_info->x;
	uint8_t mouse_y = mouse_info->y;
	uint8_t mouse_l = mouse_info->buttons[0];
	uint8_t mouse_r = mouse_info->buttons[1];

	uint16_t button_up    = KEYB_UP;
	uint16_t button_down  = KEYB_DOWN;
	uint16_t button_left  = KEYB_LEFT;
	uint16_t button_right = KEYB_RIGHT;

	uint16_t button_a     = KEYB_SPACE;
	uint16_t button_b     = KEYB_SHIFT;

	if( mouse_info == NULL ){
		return;
	}

	memset( pc60_data, 0x00, sizeof(pc60_data) );
	memset( pc60_data_4bit, 0x00, sizeof(pc60_data_4bit) );

	// A
	if( mouse_l ){
		pc60_data[usb_keytable[button_a].address] |=  (uint32_t)usb_keytable[button_a].bit;
	}
	// B
	if( mouse_r ){
		pc60_data_4bit[usb_keytable[button_b].address] |=  (uint32_t)usb_keytable[button_b].bit;
	}

	// UP
	if( mouse_y < MOUSE_UP && mouse_y > MOUSE_CENTER ){
		pc60_data[usb_keytable[button_up].address] |=  (uint32_t)usb_keytable[button_up].bit;
	}
	// DOWN
	else if( mouse_y > MOUSE_DOWN && mouse_y < MOUSE_CENTER ){
		pc60_data[usb_keytable[button_down].address] |=  (uint32_t)usb_keytable[button_down].bit;
	}
	// LEFT
	if( mouse_x < MOUSE_UP && mouse_x > MOUSE_CENTER ){
		pc60_data[usb_keytable[button_left].address] |=  (uint32_t)usb_keytable[button_left].bit;
	}
	// RIGHT
	else if( mouse_x > MOUSE_DOWN  && mouse_x < MOUSE_CENTER ){
		pc60_data[usb_keytable[button_right].address] |=  (uint32_t)usb_keytable[button_right].bit;
	}
}

static void set_usb_keycode(USBH_HandleTypeDef *phost)
{
	HID_KEYBD_Info_TypeDef	*keyb_info = USBH_HID_GetKeybdInfo(phost);

	if( keyb_info == NULL ){
		return;
	}

	memset( pc60_data, 0x00, sizeof(pc60_data) );
	memset( pc60_data_4bit, 0x00, sizeof(pc60_data_4bit) );

	// shift
	if( keyb_info->lshift || keyb_info->rshift ){
		pc60_data_4bit[usb_keytable[KEYB_SHIFT].address] |= ((uint32_t)usb_keytable[KEYB_SHIFT].bit);
	}
	// ctrl
	if( keyb_info->lctrl || keyb_info->rctrl ){
		pc60_data_4bit[usb_keytable[KEYB_CTRL].address] |= ((uint32_t)usb_keytable[KEYB_CTRL].bit);
	}
	// lalt - Graph
	if( keyb_info->lalt || keyb_info->ralt ){
		pc60_data_4bit[usb_keytable[KEYB_GRAPH].address] |= ((uint32_t)usb_keytable[KEYB_GRAPH].bit);
	}

	// USB
	for( uint32_t loop = 0; loop < 6; loop++ ){
		uint8_t		usb_code = keyb_info->keys[loop];
		if( usb_code != 0 ){
			pc60_data[usb_keytable[usb_code].address] |= ((uint32_t)usb_keytable[usb_code].bit);
		}
	}
}

static void set_usb_gamepad(USBH_HandleTypeDef *phost)
{
	HID_GAMEPAD_Info_TypeDef	*gamepad_info = USBH_HID_GetGamepadInfo(phost);

	if( gamepad_info == NULL ){
		return;
	}

	pad_con_x     = gamepad_info->x;
	pad_con_y     = gamepad_info->y;
	pad_con_hatsw = gamepad_info->hatsw;
	pad_bot_a     = gamepad_info->button[5];
	pad_bot_b     = gamepad_info->button[1];
	pad_bot_x     = gamepad_info->button[4];
	pad_bot_y     = gamepad_info->button[0];
	pad_bot_c     = gamepad_info->button[2];
	pad_bot_z     = gamepad_info->button[3];
	pad_bot_s     = gamepad_info->button[9];
	pad_bot_m     = gamepad_info->button[8];
}

static void pc60_gamepad(void)
{
	uint8_t button_up    = KEYB_UP;
	uint8_t button_down  = KEYB_DOWN;
	uint8_t button_left  = KEYB_LEFT;
	uint8_t button_right = KEYB_RIGHT;
	uint8_t button_a     = KEYB_SHIFT;
	uint8_t button_b     = KEYB_SPACE;
	uint8_t button_s     = KEYB_RET;
	uint8_t button_m     = KEYB_Y;

	if( pc60_mode != GAMEPAD_MODE ){
		return;
	}

	rapid_count++;
	if( rapid_count > RAPID_MAX )		rapid_count = 0;

	memset( pc60_data, 0x00, sizeof(pc60_data) );
	memset( pc60_data_4bit, 0x00, sizeof(pc60_data_4bit) );

	if( pad_bot_a ){
		pc60_data_4bit[usb_keytable[button_a].address] |=  (uint32_t)usb_keytable[button_a].bit;
	}
	else if( pad_bot_x && (rapid_count & 1)){
		pc60_data_4bit[usb_keytable[button_a].address] |=  (uint32_t)usb_keytable[button_a].bit;
	}
	if( pad_bot_c ){		// c
		pc60_data_4bit[usb_keytable[button_a].address] |=  (uint32_t)usb_keytable[button_a].bit;
	}
	else if( pad_bot_z && (rapid_count & 1)){
		pc60_data_4bit[usb_keytable[button_a].address] |=  (uint32_t)usb_keytable[button_a].bit;
	}
	if( pad_bot_b ){		// b
		pc60_data[usb_keytable[button_b].address] |=  (uint32_t)usb_keytable[button_b].bit;
	}
	else if( pad_bot_y && (rapid_count & 1)){
		pc60_data[usb_keytable[button_b].address] |=  (uint32_t)usb_keytable[button_b].bit;
	}

	// UP
	if( pad_con_y < 0x40 || pad_con_hatsw == HATSW_UP || pad_con_hatsw == HATSW_UPLEFT || pad_con_hatsw == HATSW_UPRIGHT ){
		pc60_data[usb_keytable[button_up].address] |=  (uint32_t)usb_keytable[button_up].bit;
	}
	// DOWN
	else if( pad_con_y > 0xc0 || pad_con_hatsw == HATSW_DOWN || pad_con_hatsw == HATSW_DOWNLEFT || pad_con_hatsw == HATSW_DOWNRIGHT ){
		pc60_data[usb_keytable[button_down].address] |=  (uint32_t)usb_keytable[button_down].bit;
	}
	// LEFT
	if( pad_con_x < 0x40 || pad_con_hatsw == HATSW_LEFT || pad_con_hatsw == HATSW_UPLEFT || pad_con_hatsw == HATSW_DOWNLEFT ){
		pc60_data[usb_keytable[button_left].address] |=  (uint32_t)usb_keytable[button_left].bit;
	}
	// RIGHT
	else if( pad_con_x > 0xc0  || pad_con_hatsw == HATSW_RIGHT || pad_con_hatsw == HATSW_UPRIGHT || pad_con_hatsw == HATSW_DOWNRIGHT ){
		pc60_data[usb_keytable[button_right].address] |=  (uint32_t)usb_keytable[button_right].bit;
	}

	if( pad_bot_s ){	// START
		pc60_data[usb_keytable[button_s].address] |=  (uint32_t)usb_keytable[button_s].bit;
	}

	if( pad_bot_m ){	// MODE
		pc60_data[usb_keytable[button_m].address] |=  (uint32_t)usb_keytable[button_m].bit;
	}
}

/**
 * @brief  EXTI line detection callbacks.
 * @param  GPIO_Pin: Specifies the pins connected EXTI line
 * @retval None
 */
void HAL_GPIO_EXTI_Callback(uint16_t GPIO_Pin)
{
	/* Prevent unused argument(s) compilation warning */
	//UNUSED(GPIO_Pin);
	/* NOTE: This function Should not be modified, when the callback is needed,
           the HAL_GPIO_EXTI_Callback could be implemented in the user file
	 */

}

/* USER CODE END 4 */

/**
  * @brief  This function is executed in case of error occurrence.
  * @retval None
  */
void Error_Handler(void)
{
  /* USER CODE BEGIN Error_Handler_Debug */
  /* User can add his own implementation to report the HAL error return state */
  __disable_irq();
  while (1)
  {
  }
  /* USER CODE END Error_Handler_Debug */
}

#ifdef  USE_FULL_ASSERT
/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t *file, uint32_t line)
{
  /* USER CODE BEGIN 6 */
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */
  /* USER CODE END 6 */
}
#endif /* USE_FULL_ASSERT */
