
#include "usbh_hid_gamepad.h"
#include "usbh_hid_parser.h"

static USBH_StatusTypeDef USBH_HID_GamepadDecode(USBH_HandleTypeDef *phost);
static void set_gamepad_value( PROPERTY_LIST index, uint8_t value );
static void init_gamepad_value(void);
static void analyze_reportdescriptor(USBH_HandleTypeDef *phost);
static void push_report_control( PROPERTY_LIST prop );
static void output_report_control(void);
static void output_report_control_sub( PROPERTY_LIST prop_id );
static void set_report_position(void);
static void init_prop_list(void);

HID_GAMEPAD_Info_TypeDef    gamepad_info;
uint32_t                    gamepad_report_data[8];
uint32_t                    gamepad_rx_report_buf[8];

static uint8_t		report_data_num     = 0;
static uint8_t		report_size_num     = 0;
static uint8_t		report_pos_num      = 0;
static uint32_t		report_logical_min  = 0;
static uint32_t		report_logical_max  = 0;
static uint32_t		report_physical_min = 0;
static uint32_t		report_physical_max = 0;
static uint8_t		report_count_num    = 0;

static uint8_t					prop_exists[PROP_MAX];
static HID_Report_ItemTypedef	prop_list[PROP_MAX];

static uint8_t			report_control_num = 0;
static PROPERTY_LIST	report_control_buffer[PROP_MAX] = {0};

USBH_StatusTypeDef USBH_HID_GamepadInit(USBH_HandleTypeDef *phost)
{
	uint32_t i;
	HID_HandleTypeDef *HID_Handle = (HID_HandleTypeDef *) phost->pActiveClass->pData;

	gamepad_info.x = 0;
	gamepad_info.y = 0;
	memset( gamepad_info.button, 0, sizeof(gamepad_info.button) );
	gamepad_info.hatsw = 0x0F;

	for (i = 0; i < (sizeof(gamepad_report_data) / sizeof(uint32_t)); i++)
	{
		gamepad_report_data[i] = 0;
		gamepad_rx_report_buf[i] = 0;
	}

	if (HID_Handle->length > sizeof(gamepad_report_data))
	{
		HID_Handle->length = sizeof(gamepad_report_data);
	}
	HID_Handle->pData = (uint8_t *)(void *)gamepad_rx_report_buf;
	USBH_HID_FifoInit(&HID_Handle->fifo, phost->device.Data, HID_QUEUE_SIZE * sizeof(gamepad_report_data));

	init_prop_list();
	analyze_reportdescriptor( phost );

	return USBH_OK;
}

HID_GAMEPAD_Info_TypeDef *USBH_HID_GetGamepadInfo(USBH_HandleTypeDef *phost)
{
	if (USBH_HID_GamepadDecode(phost) == USBH_OK)
	{
		return &gamepad_info;
	}
	else
	{
		return NULL;
	}
}

static USBH_StatusTypeDef USBH_HID_GamepadDecode(USBH_HandleTypeDef *phost)
{
	uint8_t					value = 0x00;
	HID_Report_ItemTypedef	desc;
	HID_HandleTypeDef *HID_Handle = (HID_HandleTypeDef *) phost->pActiveClass->pData;

	if (HID_Handle->length == 0)
	{
		return USBH_FAIL;
	}

	if (USBH_HID_FifoRead(&HID_Handle->fifo, &gamepad_report_data, HID_Handle->length) ==  HID_Handle->length)
	{
		init_gamepad_value();

		for( uint8_t index = 0; index < PROP_MAX; index++ ){
			if( prop_exists[index] ){
				desc.data         = prop_list[index].data;			/*data*/
				desc.size         = prop_list[index].size;			/*size*/
				desc.shift        = prop_list[index].shift;			/*shift*/
				desc.count        = prop_list[index].count;			/*count (only for array items)*/
				desc.sign         = prop_list[index].sign;			/*signed*/
				desc.logical_min  = prop_list[index].logical_min;	/*min value read can return*/
				// 苦しい処理だが、HATSWのみmax値を変更
				if( index == PROP_HATSW ){
					desc.logical_max  = 15;							/*max value read can return*/
				}
				else{
					desc.logical_max  = prop_list[index].logical_max;	/*max value read can return*/
				}
				desc.physical_min = prop_list[index].physical_min;	/*min vale device can report*/
				desc.physical_max = prop_list[index].physical_max;	/*max value device can report*/
				desc.resolution   = prop_list[index].resolution;	/*resolution*/
				value = (uint8_t)HID_ReadItem((HID_Report_ItemTypedef *) &desc, 0);
				set_gamepad_value( index, value );
			}
		}

		return USBH_OK;
	}
	return   USBH_FAIL;
}

static void set_gamepad_value( PROPERTY_LIST list_no, uint8_t value )
{
	switch( list_no )
	{
	case		PROP_X:
		gamepad_info.x = value;
		break;
	case		PROP_Y:
		gamepad_info.y = value;
		break;
	case		PROP_HATSW:
		gamepad_info.hatsw = value;
		break;
	default:
		gamepad_info.button[list_no - PROP_B1] = value;
		break;
	}
}

static void init_gamepad_value(void)
{
	// デフォルト値の設定
	gamepad_info.x = 0;
	gamepad_info.y = 0;
	gamepad_info.hatsw = 0x0F;
	for( uint8_t index = PROP_B1; index < PROP_B16; index++ ){
		gamepad_info.button[PROP_B1] = 0;
	}
}

static void analyze_reportdescriptor(USBH_HandleTypeDef *phost)
{
	uint8_t		usage_data   = 0;
	uint8_t		usage_size   = 0;
	uint8_t		usage_type   = 0;
	uint8_t		current_num  = 0;
	uint8_t		next_data1   = 0x00;
	uint8_t		next_data2   = 0x00;
	uint8_t		next_data3   = 0x00;
	uint8_t		loop         = 0;
	uint8_t		collection_num = 0;

	report_data_num     = 0;
	report_size_num     = 0;
	report_pos_num      = 0;
	report_logical_min  = 0;
	report_logical_max  = 0;
	report_physical_min = 0;
	report_physical_max = 0;
	report_count_num    = 0;

	// バッファサイズ内
	while( current_num < USBH_MAX_DATA_BUFFER ){

		// Usageの取り出し
		usage_data = phost->device.Data[current_num];

		// 終端ならば終了する
		if( usage_data == 0xC0 && collection_num < 2 ){
			break;
		}

		usage_size = (usage_data & 0x03);
		usage_type = (usage_data & 0xfc);

		next_data1 = next_data2 = next_data3 = 0;

		// サイズの取得
		switch( usage_size )
		{
		case	3:
			next_data3 = phost->device.Data[current_num + 3];
		case	2:
			next_data2 = phost->device.Data[current_num + 2];
		case	1:
			next_data1 = phost->device.Data[current_num + 1];
		}

		// Usage別の処理
		switch( usage_type ){
		case	0x04:		//Usage Page
			if( next_data1 == 0x09 ){		// (Button)
				push_report_control( PROP_B1 );
			}
			break;
		case	0x08:		//Usage
			if( next_data1 == 0x30 ){		// (X)
				push_report_control( PROP_X );
			}
			else if( next_data1 == 0x31 ){	// (Y)
				push_report_control( PROP_Y );
			}
			else if( next_data1 == 0x39 ){	// (HATSW)
				push_report_control( PROP_HATSW );
			}
			break;
		case	0x14:		//Logical Minimum
			report_logical_min = next_data1;
			if ( usage_size > 1 ){
				report_logical_min += next_data2 * 256;
			}
			if ( usage_size > 2 ){
				report_logical_min += next_data3 * 65536;
			}
			break;
		case	0x24:		//Logical Maximum
			report_logical_max = next_data1;
			if ( usage_size > 1 ){
				report_logical_max += next_data2 * 256;
			}
			if ( usage_size > 2 ){
				report_logical_max += next_data3 * 65536;
			}
			break;
		case	0x18:		//Usage Minimum
			break;
		case	0x28:		//Usage Maximum
			break;
		case	0x34:		//Physical Minimum
			report_physical_min = next_data1;
			if ( usage_size > 1 ){
				report_physical_min += next_data2 * 256;
			}
			if ( usage_size > 2 ){
				report_physical_min += next_data3 * 65536;
			}
			break;
		case	0x44:		//Physical Maximum
			report_physical_max = next_data1;
			if ( usage_size > 1 ){
				report_physical_max += next_data2 * 256;
			}
			if ( usage_size > 2 ){
				report_physical_max += next_data3 * 65536;
			}
			break;
		case	0x74:		//Report Size
			report_size_num = next_data1;
			break;
		case	0x94:		//Report Count
			report_count_num = next_data1;
			break;
		case	0x64:		//Unit
			break;
		case	0x80:		//Input
			output_report_control();
			for( loop = 0; loop < report_count_num; loop++ ){
				set_report_position();
			}
			report_count_num -= loop;
			break;
		case	0x90:		//Output
			break;
		case	0xA0:		//Collection
			collection_num++;
			break;
		case	0xB0:		//Feature
			break;
		case	0xC0:		//End Collection
			collection_num--;
			break;
		}

		current_num++;
		current_num += usage_size;

		// レポートサイズを超えたならこれ以上の分析は不要
		if( report_data_num > 7 ){
			break;
		}
	}

}

static void push_report_control( PROPERTY_LIST prop )
{
	report_control_buffer[report_control_num] = prop;
	report_control_num++;
}

static void output_report_control(void)
{
	PROPERTY_LIST	prop_id = 0;
	uint8_t			button_end = 0;

	if( report_control_num == 0 ){
		return;
	}

	// Pushした要素をセットする
	for( uint8_t index = 0; index < report_control_num; index++ ){
		prop_id = report_control_buffer[index];

		if( prop_id == PROP_B1 ){
			// ボタンの場合はCount数全てをセットする
			button_end = report_count_num;
			for( uint8_t button = 0; button < button_end; button++ ){
				output_report_control_sub( prop_id + button );
			}
		}
		else{
			// ボタン以外は個別にセット
			output_report_control_sub( prop_id );
		}
	}
	report_control_num = 0;
}

static void output_report_control_sub( PROPERTY_LIST prop_id )
{
	// レポート要素をセットする
	prop_exists[prop_id] = 1;
	prop_list[prop_id].data         = (uint8_t *)(void *)gamepad_report_data + report_data_num;
	prop_list[prop_id].size         = report_size_num;
	prop_list[prop_id].shift        = report_pos_num;
	prop_list[prop_id].logical_min  = report_logical_min;
	prop_list[prop_id].logical_max  = report_logical_max;
	prop_list[prop_id].physical_min = report_physical_min;
	prop_list[prop_id].physical_max = report_physical_max;
	set_report_position();
	report_count_num--;
}

static void set_report_position(void)
{
	// ビット位置の移動
	report_pos_num += report_size_num;

	// 8ビット以上になった場合はバイト位置の移動
	if( report_pos_num > 7 ){
		++report_data_num;
		report_pos_num -= 8;
	}
}

static void init_prop_list(void)
{
	// デフォルト値の設定
	for( uint8_t index = 0; index < PROP_MAX; index++ ){
		prop_exists[index] = 0;
		prop_list[index].data         = (uint8_t *)(void *)gamepad_report_data;	/*data*/
		prop_list[index].size         = 0;		/*size*/
		prop_list[index].shift        = 0;		/*shift*/
		prop_list[index].count        = 0;		/*count (only for array items)*/
		prop_list[index].sign         = 0;		/*signed*/
		prop_list[index].logical_min  = 0;		/*min value read can return*/
		prop_list[index].logical_max  = 255u;	/*max value read can return*/
		prop_list[index].physical_min = 0;		/*min vale device can report*/
		prop_list[index].physical_max = 255u;	/*max value device can report*/
		prop_list[index].resolution   = 1;		/*resolution*/
	}

	memset( report_control_buffer, 0, sizeof(report_control_buffer)  );
	report_control_num = 0;
}
